#!/usr/bin/env python3

# -*- encoding: utf-8; py-indent-offset: 4 -*-

# (c) Michael Honkoop <mhonkoop@comsolve.nl>

# License: GNU General Public License v2

from typing import Dict
import datetime

from cmk.agent_based.v2 import (
    AgentSection,
    CheckPlugin,
    CheckResult,
    Result,
    State,
    Metric,
    get_value_store,
)

from cmk_addons.plugins.edirectory_monitor.lib import (
    convert_timestamp,
    parse_ldap_data,
    discover_edirectory_items,
    uptime_attributes,
    time_attributes,
    time_attributes_ignored,
    non_graphable_attributes,
    total_counter_attributes,
)

def check_edirectory_items(item: str, section) -> CheckResult:
    value_store = get_value_store()
    '''Split the item key and store the last part'''
    item_detail = item.split()[-1]
    data = section.get(item)
    if not data:
        return
    for key, value in data.items():
        if (item_detail + "_" + key) in time_attributes_ignored:
            '''do not iternate keys which only hold a timestamp'''
            continue
        elif (item_detail + "_" + key) in uptime_attributes:
            '''create readable notation of uptime attributes from seconds'''
            uptime = datetime.timedelta(seconds=float(value))
            yield Result(state=State(0), summary=f"{item_detail}_{key}: {uptime}", details=f"{key}: {uptime}")
        elif (item_detail + "_" + key) in time_attributes:
            '''if a timestamp attribute is not ignored, convert it to local time in (human) readable format'''
            datevalue = convert_timestamp(value)
            yield Result(state=State(0), summary=f"{item_detail}_{key}: {datevalue}", details=f"{key}: {datevalue}")
        else:
            '''create a default result if above does not apply'''
            yield Result(state=State(0), summary=f"{item_detail}_{key}: {value}", details=f"{key}: {value}")
        if (item_detail + "_" + key) in total_counter_attributes:
            '''create a metric which is the difference between previous check value and current check value'''
            previous_value = value_store.get(key, 0)
            value_store[key] = value
            value_difference = float(value) - float(previous_value)
            yield Metric((item_detail + "_" + key), abs(value_difference), boundaries=(0, None))
        elif (item_detail + "_" + key) not in non_graphable_attributes:
            '''Only create a metric for graphable values'''
            yield Metric((item_detail + "_" + key), float(value))

agent_section_edirectory_monitor = AgentSection(
    name="edirectory_monitor",
    parse_function=parse_ldap_data,
    )

check_plugin_edirectory_monitor = CheckPlugin(
    name="edirectory_monitor",
    service_name="%s",
    discovery_function=discover_edirectory_items,
    check_function=check_edirectory_items,
)
